@@ -3,7 +3,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis-python
 #
-# Most of this work is copyright (C) 2013-2017 David R. MacIver
+# Most of this work is copyright (C) 2013-2018 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -23,7 +23,6 @@ import datetime as dt
 import operator
 from decimal import Context, Decimal
 from inspect import isclass, isfunction
-from numbers import Rational
 from fractions import Fraction
 from functools import reduce
 
@@ -33,14 +32,15 @@ from hypothesis._settings import note_deprecation
 from hypothesis.internal.cache import LRUReusedCache
 from hypothesis.searchstrategy import SearchStrategy
 from hypothesis.internal.compat import gcd, ceil, floor, hrange, \
-    text_type, integer_types, get_type_hints, getfullargspec, \
-    implements_iterator
+    text_type, get_type_hints, getfullargspec, implements_iterator
 from hypothesis.internal.floats import is_negative, float_to_int, \
     int_to_float, count_between_floats
-from hypothesis.internal.coverage import check_function
 from hypothesis.internal.renaming import renamed_arguments
 from hypothesis.utils.conventions import infer, not_set
 from hypothesis.internal.reflection import proxies, required_args
+from hypothesis.internal.validation import check_type, try_convert, \
+    check_strategy, check_valid_bound, check_valid_sizes, \
+    check_valid_integer, check_valid_interval
 
 __all__ = [
     'nothing',
@@ -117,7 +117,16 @@ def cacheable(fn):
 
 
 def base_defines_strategy(force_reusable):
+    """Returns a decorator for strategy functions.
+
+    If force_reusable is True, the generated values are assumed to be
+    reusable, i.e. immutable and safe to cache, across multiple test
+    invocations.
+
+    """
     def decorator(strategy_definition):
+        """A decorator that registers the function as a strategy and makes it
+        lazily evaluated."""
         from hypothesis.searchstrategy.lazy import LazyStrategy
         _strategies.add(strategy_definition.__name__)
 
@@ -167,7 +176,11 @@ NOTHING = Nothing()
 @cacheable
 def nothing():
     """This strategy never successfully draws a value and will always reject on
-    an attempt to draw."""
+    an attempt to draw.
+
+    Examples from this strategy do not shrink (because there are none).
+
+    """
     return NOTHING
 
 
@@ -180,15 +193,22 @@ def just(value):
     :func:`builds(callable) <hypothesis.strategies.builds>` instead
     of ``just(callable())`` to get a fresh value each time.
 
+    Examples from this strategy do not shrink (because there is only one).
+
     """
     from hypothesis.searchstrategy.misc import JustStrategy
 
     return JustStrategy(value)
 
 
-@defines_strategy
+@defines_strategy_with_reusable_values
 def none():
-    """Return a strategy which only generates None."""
+    """Return a strategy which only generates None.
+
+    Examples from this strategy do not shrink (because there is only
+    one).
+
+    """
     return just(None)
 
 
@@ -200,6 +220,17 @@ def one_of(*args):
     strategy arguments. In which case one_of(x) and one_of(\*x) are
     equivalent.
 
+    Examples from this strategy will generally shrink to ones that come from
+    strategies earlier in the list, then shrink according to behaviour of the
+    strategy that produced them. In order to get good shrinking behaviour,
+    try to put simpler strategies first. e.g. ``one_of(none(), text())`` is
+    better than ``one_of(text(), none())``.
+
+    This is especially important when using recursive strategies. e.g.
+    ``x = st.deferred(lambda: st.none() | st.tuples(x, x))`` will shrink well,
+    but ``x = st.deferred(lambda: st.tuples(x, x) | st.none())`` will shrink
+    very badly indeed.
+
     """
     if len(args) == 1 and not isinstance(args[0], SearchStrategy):
         try:
@@ -219,6 +250,9 @@ def integers(min_value=None, max_value=None):
     If min_value is not None then all values will be >= min_value. If
     max_value is not None then all values will be <= max_value
 
+    Examples from this strategy will shrink towards zero, and negative values
+    will also shrink towards positive (i.e. -n may be replaced by +n).
+
     """
 
     check_valid_bound(min_value, 'min_value')
@@ -264,7 +298,12 @@ def integers(min_value=None, max_value=None):
 @cacheable
 @defines_strategy
 def booleans():
-    """Returns a strategy which generates instances of bool."""
+    """Returns a strategy which generates instances of bool.
+
+    Examples from this strategy will shrink towards False (i.e.
+    shrinking will try to replace True with False where possible).
+
+    """
     from hypothesis.searchstrategy.misc import BoolStrategy
     return BoolStrategy()
 
@@ -286,6 +325,11 @@ def floats(
     Where not explicitly ruled out by the bounds, all of infinity, -infinity
     and NaN are possible values generated by this strategy.
 
+    Examples from this strategy have a complicated and hard to explain
+    shrinking behaviour, but it tries to improve "human readability". Finite
+    numbers will be preferred to infinity and infinity will be preferred to
+    NaN.
+
     """
 
     if allow_nan is None:
@@ -381,7 +425,12 @@ def floats(
 @cacheable
 @defines_strategy_with_reusable_values
 def complex_numbers():
-    """Returns a strategy that generates complex numbers."""
+    """Returns a strategy that generates complex numbers.
+
+    Examples from this strategy shrink by shrinking their component real
+    and imaginary parts.
+
+    """
     from hypothesis.searchstrategy.numbers import ComplexStrategy
     return ComplexStrategy(
         tuples(floats(), floats())
@@ -397,6 +446,8 @@ def tuples(*args):
     e.g. tuples(integers(), integers()) would generate a tuple of length
     two with both values an integer.
 
+    Examples from this strategy shrink by shrinking their component parts.
+
     """
     for arg in args:
         check_strategy(arg)
@@ -416,6 +467,11 @@ def sampled_from(elements):
     :class:`~python:enum.Enum` objects.  :class:`~python:enum.Flag` objects
     may also generate any combination of their members.
 
+    Examples from this strategy shrink by replacing them with values earlier in
+    the list. So e.g. sampled_from((10, 1)) will shrink by trying to replace
+    1 values with 10, and sampled_from((1, 10)) will shrink by trying to
+    replace 10 values with 1.
+
     """
     from hypothesis.searchstrategy.misc import SampledFromStrategy
     from hypothesis.internal.conjecture.utils import check_sample
@@ -433,6 +489,9 @@ def sampled_from(elements):
     return SampledFromStrategy(values)
 
 
+_AVERAGE_LIST_LENGTH = 5.0
+
+
 @cacheable
 @defines_strategy
 def lists(
@@ -456,6 +515,9 @@ def lists(
     when given a value drawn from elements. The resulting list will satisfy the
     condition that for i != j, unique_by(result[i]) != unique_by(result[j]).
 
+    Examples from this strategy shrink by trying to remove elements from the
+    list, and by shrinking each individual element of the list.
+
     """
     check_valid_sizes(min_size, average_size, max_size)
     if elements is None or (max_size is not None and max_size <= 0):
@@ -476,22 +538,24 @@ def lists(
             def unique_by(x):
                 return x
 
+    if min_size is None:
+        min_size = 0
+
+    if max_size is None:
+        max_size = float('inf')
+
+    if average_size is None:
+        average_size = max(
+            _AVERAGE_LIST_LENGTH,
+            min_size * 2
+        )
+        if max_size < float('inf'):
+            average_size = min(average_size, 0.5 * (min_size + max_size))
+
+    from hypothesis.searchstrategy.collections import ListStrategy, \
+        UniqueListStrategy
     if unique_by is not None:
-        from hypothesis.searchstrategy.collections import UniqueListStrategy
-        min_size = min_size or 0
-        max_size = max_size or float(u'inf')
-        if average_size is None:
-            if max_size < float(u'inf'):
-                if max_size <= 5:
-                    average_size = min_size + 0.75 * (max_size - min_size)
-                else:
-                    average_size = (max_size + min_size) / 2
-            else:
-                average_size = max(
-                    _AVERAGE_LIST_LENGTH,
-                    min_size * 2
-                )
-        result = UniqueListStrategy(
+        return UniqueListStrategy(
             elements=elements,
             average_size=average_size,
             max_size=max_size,
@@ -499,20 +563,10 @@ def lists(
             key=unique_by
         )
     else:
-        from hypothesis.searchstrategy.collections import ListStrategy
-        if min_size is None:
-            min_size = 0
-        if average_size is None:
-            if max_size is None:
-                average_size = _AVERAGE_LIST_LENGTH
-            else:
-                average_size = (min_size + max_size) * 0.5
-
-        result = ListStrategy(
+        return ListStrategy(
             (elements,), average_length=average_size,
             min_size=min_size, max_size=max_size,
         )
-    return result
 
 
 @cacheable
@@ -524,6 +578,9 @@ def sets(elements=None, min_size=None, average_size=None, max_size=None):
     are hashable until running the test, so you can define a strategy
     for sets of an unhashable type but it will fail at test time.
 
+    Examples from this strategy shrink by trying to remove elements from the
+    set, and by shrinking each individual element of the set.
+
     """
     return lists(
         elements=elements, min_size=min_size, average_size=average_size,
@@ -584,6 +641,9 @@ def fixed_dictionaries(mapping):
     instance of OrderedDict the keys will also be in the same order,
     otherwise the order is arbitrary.
 
+    Examples from this strategy shrink by shrinking each individual value in
+    the generated dictionary.
+
     """
     from hypothesis.searchstrategy.collections import FixedKeysDictStrategy
     check_type(dict, mapping, 'mapping')
@@ -603,6 +663,9 @@ def dictionaries(
 
     The size parameters have the same interpretation as for lists.
 
+    Examples from this strategy shrink by trying to remove keys from the
+    generated dictionary, and by shrinking each generated key and value.
+
     """
     check_valid_sizes(min_size, average_size, max_size)
     if max_size == 0:
@@ -625,6 +688,8 @@ def streaming(elements):
     The result is iterable (the iterator will never terminate) and
     indexable.
 
+    Examples from this strategy shrink by trying to shrink each value drawn.
+
     .. deprecated:: 3.15.0
         Use :func:`data() <hypothesis.strategies.data>` instead.
 
@@ -669,6 +734,8 @@ def characters(whitelist_categories=None, blacklist_categories=None,
     will be not be produced. Any overlap between ``whitelist_characters`` and
     ``blacklist_characters`` will raise an exception.
 
+    Examples from this strategy shrink towards smaller codepoints.
+
     """
     if (
         min_codepoint is not None and max_codepoint is not None and
@@ -724,11 +791,15 @@ def text(
     """Generates values of a unicode text type (unicode on python 2, str on
     python 3) with values drawn from alphabet, which should be an iterable of
     length one strings or a strategy generating such. If it is None it will
-    default to generating the full unicode range. If it is an empty collection
-    this will only generate empty strings.
+    default to generating the full unicode range (excluding surrogate
+    characters). If it is an empty collection this will only generate empty
+    strings.
 
     min_size, max_size and average_size have the usual interpretations.
 
+    Examples from this strategy shrink towards shorter strings, and with the
+    characters in the text shrinking as per the alphabet strategy.
+
     """
     from hypothesis.searchstrategy.strings import StringStrategy
     if alphabet is None:
@@ -776,6 +847,9 @@ def from_regex(regex):
     string, while ``"."`` will return any string, and ``r"\\A.$"`` will return
     a single character optionally followed by a ``"\\n"``.
 
+    Examples from this strategy shrink towards shorter strings and lower
+    character values.
+
     """
     from hypothesis.searchstrategy.regex import regex_strategy
     return regex_strategy(regex)
@@ -791,6 +865,9 @@ def binary(
 
     min_size, average_size and max_size have the usual interpretations.
 
+    Examples from this strategy shrink towards smaller strings and lower byte
+    values.
+
     """
     from hypothesis.searchstrategy.strings import BinaryStringStrategy, \
         FixedSizeBytes
@@ -809,7 +886,11 @@ def binary(
 @defines_strategy
 def randoms():
     """Generates instances of Random (actually a Hypothesis specific
-    RandomWithSeed class which displays what it was initially seeded with)"""
+    RandomWithSeed class which displays what it was initially seeded with)
+
+    Examples from this strategy shrink to seeds closer to zero.
+
+    """
     from hypothesis.searchstrategy.misc import RandomStrategy
     return RandomStrategy(integers())
 
@@ -836,38 +917,61 @@ def random_module():
     a random number generator you should use the randoms() strategy
     which will give you one.
 
+    Examples from these strategy shrink to seeds closer to zero.
+
     """
     from hypothesis.control import cleanup
     import random
 
-    def seed_random(seed):
-        state = random.getstate()
-        random.seed(seed)
-        cleanup(lambda: random.setstate(state))
-        return RandomSeeder(seed)
+    class RandomModule(SearchStrategy):
+        def do_draw(self, data):
+            data.can_reproduce_example_from_repr = False
+            seed = data.draw(integers())
+            state = random.getstate()
+            random.seed(seed)
+            cleanup(lambda: random.setstate(state))
+            return RandomSeeder(seed)
 
-    return shared(
-        integers().map(seed_random),
-        'hypothesis.strategies.random_module()',
-    )
+    return shared(RandomModule(), 'hypothesis.strategies.random_module()')
 
 
 @cacheable
 @defines_strategy
-def builds(target, *args, **kwargs):
+def builds(*callable_and_args, **kwargs):
     """Generates values by drawing from ``args`` and ``kwargs`` and passing
-    them to ``target`` in the appropriate argument position.
+    them to the callable (provided as the first positional argument) in the
+    appropriate argument position.
 
     e.g. ``builds(target, integers(), flag=booleans())`` would draw an
     integer ``i`` and a boolean ``b`` and call ``target(i, flag=b)``.
 
-    If ``target`` has type annotations, they will be used to infer a strategy
+    If the callable has type annotations, they will be used to infer a strategy
     for required arguments that were not passed to builds.  You can also tell
     builds to infer a strategy for an optional argument by passing the special
     value :const:`hypothesis.infer` as a keyword argument to
-    builds, instead of a strategy for that argument to ``target``.
+    builds, instead of a strategy for that argument to the callable.
+
+    Examples from this strategy shrink by shrinking the argument values to
+    the callable.
 
     """
+    if callable_and_args:
+        target, args = callable_and_args[0], callable_and_args[1:]
+        if not callable(target):
+            raise InvalidArgument(
+                'The first positional argument to builds() must be a callable '
+                'target to construct.')
+    elif 'target' in kwargs and callable(kwargs['target']):
+        args = []
+        note_deprecation(
+            'Specifying the target as a keyword argument to builds() is '
+            'deprecated. Provide it as the first positional argument instead.')
+        target = kwargs.pop('target')
+    else:
+        raise InvalidArgument(
+            'builds() must be passed a callable as the first positional '
+            'argument, but no positional arguments were given.')
+
     if infer in args:
         # Avoid an implementation nightmare juggling tuples and worse things
         raise InvalidArgument('infer was passed as a positional argument to '
@@ -887,30 +991,16 @@ def builds(target, *args, **kwargs):
     )
 
 
-def delay_error(func):
-    """A decorator to make exceptions lazy but success immediate.
-
-    We want from_type to resolve to a strategy immediately if possible,
-    for a useful repr and interactive use, but delay errors until a
-    value would be drawn to localise them to a particular test.
-
-    """
+def _defer_from_type(func):
+    """Decorator to make from_type lazy to support recursive definitions."""
     @proxies(func)
     def inner(*args, **kwargs):
-        try:
-            return func(*args, **kwargs)
-        except Exception as e:
-            error = e
-
-            def lazy_error():
-                raise error
-
-            return builds(lazy_error)
+        return deferred(lambda: func(*args, **kwargs))
     return inner
 
 
 @cacheable
-@delay_error
+@_defer_from_type
 def from_type(thing):
     """Looks up the appropriate search strategy for the given type.
 
@@ -939,12 +1029,12 @@ def from_type(thing):
 
     """
     from hypothesis.searchstrategy import types
-    if not isinstance(thing, type):
-        try:
+    try:
+        import typing
+        if not isinstance(thing, type):
             # At runtime, `typing.NewType` returns an identity function rather
             # than an actual type, but we can check that for a possible match
             # and then read the magic attribute to unwrap it.
-            import typing
             if all([
                 hasattr(thing, '__supertype__'), hasattr(typing, 'NewType'),
                 isfunction(thing), getattr(thing, '__module__', 0) == 'typing'
@@ -955,8 +1045,16 @@ def from_type(thing):
             if getattr(thing, '__origin__', None) is typing.Union:
                 args = sorted(thing.__args__, key=types.type_sorting_key)
                 return one_of([from_type(t) for t in args])
-        except ImportError:  # pragma: no cover
-            pass
+        # We can't resolve forward references, and under Python 3.5 (only)
+        # a forward reference is an instance of type.  Hence, explicit check:
+        elif type(thing) == typing._ForwardRef:  # pragma: no cover
+            raise ResolutionFailed(
+                'thing=%s cannot be resolved.  Upgrading to python>=3.6 may '
+                'fix this problem via improvements to the typing module.'
+                % (thing,))
+    except ImportError:  # pragma: no cover
+        pass
+    if not isinstance(thing, type):
         raise InvalidArgument('thing=%s must be a type' % (thing,))
     # Now that we know `thing` is a type, the first step is to check for an
     # explicitly registered strategy.  This is the best (and hopefully most
@@ -1034,6 +1132,9 @@ def fractions(min_value=None, max_value=None, max_denominator=None):
     values is no greater than max_denominator. Note that max_denominator must
     be None or a positive integer.
 
+    Examples from this strategy shrink towards smaller denominators, then
+    closer to zero.
+
     """
     min_value = try_convert(Fraction, min_value, 'min_value')
     max_value = try_convert(Fraction, max_value, 'max_value')
@@ -1137,6 +1238,9 @@ def decimals(min_value=None, max_value=None,
     If ``places`` is not None, all finite values drawn from the strategy will
     have that number of digits after the decimal place.
 
+    Examples from this strategy do not have a well defined shrink order but
+    try to maximize human readability when shrinking.
+
     """
     # Convert min_value and max_value to Decimal values, and validate args
     check_valid_integer(places)
@@ -1229,6 +1333,10 @@ def recursive(base, extend, max_leaves=100):
     So e.g. ``False``, ``[True]``, ``[False, []]``, and ``[[[[True]]]]`` are
     all valid values to be drawn from that strategy.
 
+    Examples from this strategy shrink by trying to reduce the amount of
+    recursion and by shrinking according to the shrinking behaviour of base
+    and the result of extend.
+
     """
 
     from hypothesis.searchstrategy.recursive import RecursiveStrategy
@@ -1238,7 +1346,12 @@ def recursive(base, extend, max_leaves=100):
 @defines_strategy
 def permutations(values):
     """Return a strategy which returns permutations of the collection
-    ``values``."""
+    ``values``.
+
+    Examples from this strategy shrink by trying to become closer to the
+    original order of values.
+
+    """
     from hypothesis.internal.conjecture.utils import integer_range
 
     values = list(values)
@@ -1294,6 +1407,8 @@ def datetimes(
     :py:func:`sampled_from`.  Ensure that simple values such as None or UTC
     are at the beginning of the list for proper minimisation.
 
+    Examples from this strategy shrink towards midnight on January 1st 2000.
+
     """
     # Why must bounds be naive?  In principle, we could also write a strategy
     # that took aware bounds, but the API and validation is much harder.
@@ -1332,7 +1447,11 @@ def dates(
     min_value=dt.date.min, max_value=dt.date.max,
     min_date=None, max_date=None,
 ):
-    """A strategy for dates between ``min_date`` and ``max_date``."""
+    """A strategy for dates between ``min_date`` and ``max_date``.
+
+    Examples from this strategy shrink towards January 1st 2000.
+
+    """
     from hypothesis.searchstrategy.datetime import DateStrategy
 
     check_type(dt.date, min_value, 'min_value')
@@ -1356,6 +1475,9 @@ def times(
 
     The ``timezones`` argument is handled as for :py:func:`datetimes`.
 
+    Examples from this strategy shrink towards midnight, with the timezone
+    component shrinking as for the strategy that provided it.
+
     """
     check_type(dt.time, min_value, 'min_value')
     check_type(dt.time, max_value, 'max_value')
@@ -1379,7 +1501,11 @@ def timedeltas(
     min_value=dt.timedelta.min, max_value=dt.timedelta.max,
     min_delta=None, max_delta=None
 ):
-    """A strategy for timedeltas between ``min_value`` and ``max_value``."""
+    """A strategy for timedeltas between ``min_value`` and ``max_value``.
+
+    Examples from this strategy shrink towards zero.
+
+    """
     from hypothesis.searchstrategy.datetime import TimedeltaStrategy
 
     check_type(dt.timedelta, min_value, 'min_value')
@@ -1399,9 +1525,13 @@ def composite(f):
     :ref:`the full documentation for more details <composite-strategies>`
     about how to use this function.
 
+    Examples from this strategy shrink by shrinking the output of each draw
+    call.
+
     """
 
     from hypothesis.internal.reflection import define_function_signature
+    from hypothesis.internal.conjecture.utils import calc_label_from_cls
     argspec = getfullargspec(f)
 
     if (
@@ -1420,6 +1550,8 @@ def composite(f):
               if k in (argspec.args + argspec.kwonlyargs + ['return'])}
     new_argspec = argspec._replace(args=argspec.args[1:], annotations=annots)
 
+    label = calc_label_from_cls(f)
+
     @defines_strategy
     @define_function_signature(f.__name__, f.__doc__, new_argspec)
     def accept(*args, **kwargs):
@@ -1433,6 +1565,10 @@ def composite(f):
                     return data.draw(strategy)
 
                 return f(draw, *args, **kwargs)
+
+            @property
+            def label(self):
+                return label
         return CompositeStrategy()
     accept.__module__ = f.__module__
     return accept
@@ -1453,6 +1589,8 @@ def shared(base, key=None):
     >>> x = shared(s, key="hi")
     >>> y = shared(s, key="hi")
 
+    Examples from this strategy shrink as per their base strategy.
+
     """
     from hypothesis.searchstrategy.shared import SharedStrategy
     return SharedStrategy(base, key)
@@ -1469,6 +1607,9 @@ def choices():
         Use :func:`data() <hypothesis.strategies.data>` with
         :func:`sampled_from() <hypothesis.strategies.sampled_from>` instead.
 
+    Examples from this strategy shrink by making each choice function return
+    an earlier value in the sequence passed to it.
+
     """
     from hypothesis.control import note, current_build_context
     from hypothesis.internal.conjecture.utils import choice, check_sample
@@ -1501,6 +1642,7 @@ def choices():
         supports_find = False
 
         def do_draw(self, data):
+            data.can_reproduce_example_from_repr = False
             return Chooser(current_build_context(), data)
 
     return shared(
@@ -1521,6 +1663,8 @@ def uuids(version=None):
     All returned values from this will be unique, so e.g. if you do
     ``lists(uuids())`` the resulting list will never contain duplicates.
 
+    Examples from this strategy don't have any meaningful shrink order.
+
     """
     from uuid import UUID
     if version not in (None, 1, 2, 3, 4, 5):
@@ -1542,6 +1686,8 @@ def runner(default=not_set):
     If there is no current test runner and a default is provided, return
     that default. If no default is provided, raises InvalidArgument.
 
+    Examples from this strategy do not shrink (because there is only one).
+
     """
     class RunnerStrategy(SearchStrategy):
 
@@ -1572,6 +1718,9 @@ def data():
     See :ref:`the rest of the documentation <interactive-draw>` for more
     complete information.
 
+    Examples from this strategy do not shrink (because there is only one),
+    but the result of calls to each draw() call shrink as they normally would.
+
     """
     from hypothesis.control import note
 
@@ -1597,6 +1746,8 @@ def data():
         supports_find = False
 
         def do_draw(self, data):
+            data.can_reproduce_example_from_repr = False
+
             if not hasattr(data, 'hypothesis_shared_data_strategy'):
                 data.hypothesis_shared_data_strategy = DataObject(data)
             return data.hypothesis_shared_data_strategy
@@ -1652,131 +1803,6 @@ def register_type_strategy(custom_type, strategy):
     types._global_type_lookup[custom_type] = strategy
     from_type.__clear_cache()
 
-# Private API below here
-
-
-@check_function
-def check_type(typ, arg, name=''):
-    if name:
-        name += '='
-    if not isinstance(arg, typ):
-        if isinstance(typ, type):
-            typ_string = typ.__name__
-        else:
-            typ_string = 'one of %s' % (
-                ', '.join(t.__name__ for t in typ))
-        raise InvalidArgument('Expected %s but got %s%r (type=%s)'
-                              % (typ_string, name, arg, type(arg).__name__))
-
-
-@check_function
-def check_strategy(arg, name=''):
-    check_type(SearchStrategy, arg, name)
-
-
-@check_function
-def check_valid_integer(value):
-    """Checks that value is either unspecified, or a valid integer.
-
-    Otherwise raises InvalidArgument.
-
-    """
-    if value is None:
-        return
-    check_type(integer_types, value)
-
-
-@check_function
-def check_valid_bound(value, name):
-    """Checks that value is either unspecified, or a valid interval bound.
-
-    Otherwise raises InvalidArgument.
-
-    """
-    if value is None or isinstance(value, integer_types + (Rational,)):
-        return
-    if math.isnan(value):
-        raise InvalidArgument(u'Invalid end point %s=%r' % (name, value))
-
-
-@check_function
-def try_convert(typ, value, name):
-    if value is None:
-        return None
-    if isinstance(value, typ):
-        return value
-    try:
-        return typ(value)
-    except TypeError:
-        raise InvalidArgument(
-            'Cannot convert %s=%r of type %s to type %s' % (
-                name, value, type(value).__name__, typ.__name__
-            )
-        )
-    except (OverflowError, ValueError):
-        raise InvalidArgument(
-            'Cannot convert %s=%r to type %s' % (
-                name, value, typ.__name__
-            )
-        )
-
-
-@check_function
-def check_valid_size(value, name):
-    """Checks that value is either unspecified, or a valid non-negative size
-    expressed as an integer/float.
-
-    Otherwise raises InvalidArgument.
-
-    """
-    if value is None:
-        return
-    check_type(integer_types + (float,), value)
-    if value < 0:
-        raise InvalidArgument(u'Invalid size %s %r < 0' % (value, name))
-    if isinstance(value, float) and math.isnan(value):
-        raise InvalidArgument(u'Invalid size %s %r' % (value, name))
-
-
-@check_function
-def check_valid_interval(lower_bound, upper_bound, lower_name, upper_name):
-    """Checks that lower_bound and upper_bound are either unspecified, or they
-    define a valid interval on the number line.
-
-    Otherwise raises InvalidArgument.
-
-    """
-    if lower_bound is None or upper_bound is None:
-        return
-    if upper_bound < lower_bound:
-        raise InvalidArgument(
-            'Cannot have %s=%r < %s=%r' % (
-                upper_name, upper_bound, lower_name, lower_bound
-            ))
-
-
-@check_function
-def check_valid_sizes(min_size, average_size, max_size):
-    check_valid_size(min_size, 'min_size')
-    check_valid_size(max_size, 'max_size')
-    check_valid_size(average_size, 'average_size')
-    check_valid_interval(min_size, max_size, 'min_size', 'max_size')
-    check_valid_interval(average_size, max_size, 'average_size', 'max_size')
-    check_valid_interval(min_size, average_size, 'min_size', 'average_size')
-
-    if average_size is not None:
-        if (
-            (max_size is None or max_size > 0) and
-            average_size is not None and average_size <= 0.0
-        ):
-            raise InvalidArgument(
-                'Cannot have average_size=%r < min_size=%r' % (
-                    average_size, min_size
-                ))
-
-
-_AVERAGE_LIST_LENGTH = 5.0
-
 
 @cacheable
 def deferred(definition):
@@ -1793,7 +1819,7 @@ def deferred(definition):
     >>> import hypothesis.strategies as st
     >>> x = st.deferred(lambda: st.booleans() | st.tuples(x, x))
     >>> x.example()
-    (False, (False, True))
+    (((False, (True, True)), (False, True)), (True, True))
     >>> x.example()
     True
 
@@ -1802,9 +1828,12 @@ def deferred(definition):
     >>> a = st.deferred(lambda: st.booleans() | b)
     >>> b = st.deferred(lambda: st.tuples(a, a))
     >>> a.example()
-    (((True, True), False), True)
+    True
     >>> b.example()
-    (((( False, ((True, False), (True, True))), True), False), True)
+    (False, (False, ((False, True), False)))
+
+    Examples from this strategy shrink as they normally would from the strategy
+    returned by the definition.
 
     """
     from hypothesis.searchstrategy.deferred import DeferredStrategy
